import java.awt.*;
import java.awt.geom.*;

/**
 * <b>Programm:</b> WaveGradient<br>
 * <b>Copyright:</b> 2002 Andreas Gohr, Frank Schubert<br>
 * <b>License:</b> GPL2 or higher<br>
 * <br>
 * <b>Info:</b> holds data for a monochromatic wave
 */
 
public class MonoWave{
  double _A;
  double _f;
  double _phi;
  double _lambda;
  double _scale;
  double _rotation;
  MonoWaveSlider _mwslider = null;
  GeneralPath _vpath;
  GeneralPath _hpath;

  /**
   * Constructor
   */
  public MonoWave(double A,double f,double lambda){
    //set up defaults:
    _mwslider = new MonoWaveSlider();
    _mwslider.getAmplitudeSlider().setDoubleMinimum(0.0);
    _mwslider.getAmplitudeSlider().setDoubleMaximum(20.0);
    _mwslider.getFrequencySlider().setDoubleMinimum(0.0);
    _mwslider.getFrequencySlider().setDoubleMaximum(2.0);
    _mwslider.getWaveLengthSlider().setDoubleMinimum(0.0);
    _mwslider.getWaveLengthSlider().setDoubleMaximum(20.0);
    _mwslider.getPhaseSlider().setDoubleMinimum(-2.0*Math.PI);
    _mwslider.getPhaseSlider().setDoubleMaximum( 2.0*Math.PI);
    
    setAmplitude(A);
    setFrequency(f);
    setWaveLength(lambda);
    setPhase(0.0);
    setScale(50.0);
    setRotationRad(0.0);
    _vpath = new GeneralPath();
  }
  
  /**
   * sets the GeneralPath for vertical projection
   */
   
  public void setVerticalPath(GeneralPath gp){
    _vpath = gp;
  }
  
  /**
   * sets the GeneralPath for horizontal projection
   */
   
  public void setHorizontalPath(GeneralPath gp){
    _hpath = gp;
  }
  
  /**
   * returns GeneralPath for vertical projection
   */
   
  public GeneralPath getVerticalPath(){
    return _vpath;
  }
  
  /**
   * returns GeneralPath for horizontal projection
   */
   
  public GeneralPath getHorizontalPath(){
    return _hpath;
  }
  /**
   * Return MonoWaveSliderobject
   */
  public MonoWaveSlider getMonoWaveSlider(){
    return _mwslider;
  }


  /**
   * sets amplitude
   */
  public void setAmplitude(double A) {
    _A = A;
    _mwslider.getAmplitudeSlider().setDoubleValue(A);
  }
  
  /**
   * sets wavelength
   */
  public void setWaveLength(double lambda) {
    _lambda = lambda;
    _mwslider.getWaveLengthSlider().setDoubleValue(lambda);
  }
  
  /**
   * sets phaseshift
   */
  public void setPhase(double phi) {
    _phi = phi;
    _mwslider.getPhaseSlider().setDoubleValue(phi);
  }

  /**
   * sets frequency
   */
  public void setFrequency(double f) {
    _f = f;
    _mwslider.getFrequencySlider().setDoubleValue(f);
  }
  
  /**
   * sets scale
   */
  public void setScale(double scale) {
    _scale = scale;
  }

  /**
   * sets rotation in Degrees
   */
  public void setRotationDeg(double rotation) {
    rotation = rotation % 360;
    _rotation = (rotation*2*Math.PI)/360;
  }

  /**
   * sets rotation in Radians
   */
  public void setRotationRad(double rotation) {
    rotation = rotation % (2*Math.PI);
    _rotation = rotation;
  }


  /**
   * gets amplitude
   */
  public double getAmplitude() {
    return(_A);
  }
  
  /**
   * gets wavelength
   */
  public double getWaveLength() {
    return (_lambda);
  }
  
  /**
   * gets phaseshift
   */
  public double getPhase() {
    return (_phi);
  }
  
  /**
   * gets frequency
   */
  public double getFrequency() {
    return(_f);
  }

  /**
   * gets circlefrequency (omega)
   */
  public double getCircleFrequency(){
    return (2*Math.PI*_f);
  }
 
  /**
   * gets Wavekey (k)
   */
  public double getWaveKey(){
    return (2*Math.PI/_lambda);
  }
  
  /**
   * gets Psi
   */
  public double getPsi(double x,double t) {
    x = x/_scale;
    t = t/_scale;
    
    return(_A*Math.sin( getWaveKey()*x - getCircleFrequency()*t + getPhase()));
  }
  
  /**
   * gets scale
   */
  public double getScale() {
    return(_scale); 
  }

  /**
   * gets rotation in degrees
   */
  public double getRotationDeg() {
    return( (360*_rotation) / (2*Math.PI) ); 
  }

  /**
   * gets rotation in radians
   */
  public double getRotationRad() {
    return(_rotation); 
  }

  
  /**
   * gets ratio between SliderMaximum and Amplitude
   */
  public double getRatio() {
    return(_A/_mwslider.getAmplitudeSlider().getDoubleMaximum()); 
  }

}
